#!/bin/sh
############################################################################
# SCRIPT:	powermon.sh
#
# DESCRIPTION:
# 
# Currently a userspace process (not yet a daemon) that will:
#
# 1. Map the GPIO input pin specified by GPIO_PIN number via sysfs as
#    an edge according to 
# 
# NOTES: 
# 1. The sysfs interface will be removed from the kernel starting in 2020.
# Will need to utilize the character device ABI instead for GPIO.
#
############################################################################

# Internal variables - DO NOT MODIFY
POWERMON_CONFIG_FILE=/etc/default/powermon.conf

# powermon.conf defaults - DO NOT MODIFY
DEFAULT_GPIO_PIN=148
DEFAULT_POWER_BUTTON_ACTIVE_VALUE=0


STORED_BRIGHTNESS_UDEV=/var/lib/systemd/backlight/platform-backlight\:backlight\:backlight
BRIGHTNESS_UDEV=/sys/class/backlight/backlight/brightness
BACKLIGHT_VALUE=`cat $BRIGHTNESS_UDEV`
MAX_BACKLIGHT_VALUE=`cat /sys/class/backlight/backlight/max_brightness`

############################################################################
#
# FUNCTION:	do_check_backlight_restore()
#
# DESCRIPTION:	Checks if the backlight brightness level is set to 1 (i.e.
#		from a power fail to preserve energy. Need to do this 
#		to restore backlight brightness to max after first reboot
#		after a power failure. This is because this script turns 
#		backlight brightness to 0, and before shutdown is complete
#		systemd-backlight@.service saves the backlight brightness
#		value (clamped to 1, see man systemd-backlight). 
#
#		If brightness <= 1, change it to max and restore that value
#		so it is saved before the second reboot after a power 
#		failure.
#
############################################################################
do_check_backlight_restore() {

if [ $BACKLIGHT_VALUE -le "1" ]; then
        echo "Adjusting brightness from "$BACKLIGHT_VALUE" to "$MAX_BACKLIGHT_VALUE
        echo $MAX_BACKLIGHT_VALUE > $BRIGHTNESS_UDEV

	echo "Storing new brightness value"
        echo $MAX_BACKLIGHT_VALUE > $STORED_BRIGHTNESS_UDEV
fi
} # do_check_backlight_restore()

############################################################################
#
# FUNCTION:	do_check_config()
#
# DESCRIPTION:	Checks if conf file exists and loads defaults if not found
#		or if variables not set
#
############################################################################
do_check_config() {
	if [ -f $POWERMON_CONFIG_FILE ]; then
		echo "Found "$POWERMON_CONFIG_FILE

		#
		# Set the input field separator (IFS) to be "=", then
		# read like a key=value pair for each line in config.
		# Export the variables locally then determine if they
		# are valid or to use the defaults
		#

		IFS="="
		while read var value
		do
			echo "$var=$value"
			export "$var"="$value"
		done < $POWERMON_CONFIG_FILE
	else
		echo $POWERMON_CONFIG_FILE" not found"
	fi

	if [ $CONF_GPIO_PIN ]; then
		echo "Configuration GPIO Pin Exists"
		GPIO_PIN=$CONF_GPIO_PIN
	else
		echo "Configuration GPIO Pin does not exist"
		GPIO_PIN=$DEFAULT_GPIO_PIN
	fi

	if [ $CONF_POWER_BUTTON_ACTIVE_VALUE ]; then
		POWER_BUTTON_ACTIVE_VALUE=$CONF_POWER_BUTTON_ACTIVE_VALUE
	else
		POWER_BUTTON_ACTIVE_VALUE=$DEFAULT_POWER_BUTTON_ACTIVE_VALUE
	fi

	return
} # do_check_config

############################################################################
#
# FUNCTION:	do_make_power_button()
#
# DESCRIPTION:	Creates GPIO input "power button" just used as GPIO, not 
#		utilizing gpio-keys in DT.
#
############################################################################
do_make_power_button() {

	echo $GPIO_PIN > /sys/class/gpio/export

	#
	# sysfs gpio edge needs specified to be able to monitor its event
	#
	if [ $POWER_BUTTON_ACTIVE_VALUE -eq "1" ]; then
		echo rising > $POWER_BUTTON_EDGE
	else
		echo falling > $POWER_BUTTON_EDGE
	fi

	return
} # do_make_power_button

############################################################################
#
# FUNCTION:	do_check_power_button_exists()
#
# DESCRIPTION:	Checks if power button has been created in sysfs gpio and 
#		creates if not present.		
#
############################################################################
do_check_power_button_exists() {

POWER_BUTTON_BASE=/sys/class/gpio/gpio"$GPIO_PIN"
POWER_BUTTON="$POWER_BUTTON_BASE"/value
POWER_BUTTON_EDGE="$POWER_BUTTON_BASE"/edge

	echo "Checking if "$POWER_BUTTON" exists"
	if [ ! -f $POWER_BUTTON ]; then
		echo $POWER_BUTTON" does not exist. Creating..."
		do_make_power_button
	fi
	return
} # do_check_power_button_exists

############################################################################
#
# FUNCTION:	do_notify_and_shutdown()
#
# DESCRIPTION:	Shutdown system
#
############################################################################
do_notify_and_shutdown() {
	echo "Changing backlight brightness to 3 to save remaining power, but show message on screen"
	echo 3 > /sys/class/backlight/backlight/brightness
    systemctl stop ttru3
    echo "Wait"
    sleep 3s
	echo "Changing backlight brightness to 0 to save remaining power"
    echo 0 > /sys/class/backlight/backlight/brightness
	echo "Shutting down..."
	# Sends SIGTERM, waits default 5 seconds before sending SIGHUP
	# IMPORTANT NOTE: It is unknown if there is a default order in which SIGTERM 
	# is sent by systemctl, meaning if there are dependencies on the GUI application
	# it may need to be handled separately from the entire system so no 
	# dependencies are terminated (i.e. USB), which may make a graceful shutdown
	# moot.

	shutdown now
	exit 0;
}
# do_notify_and_shutdown

############################################################################
#	MAIN SCRIPT
############################################################################

do_check_backlight_restore
do_check_config
do_check_power_button_exists

echo "Power fail monitor. Watching:"$POWER_BUTTON

#
# Monitors the file $POWER_BUTTON to be modified to contain
# $POWER_BUTTON_ACTIVE_VALUE before declaring a power failure
# in progress before `do_notify_and_shutdown` is called
#

while inotifywait -e modify $POWER_BUTTON; do
	if grep -q $POWER_BUTTON_ACTIVE_VALUE $POWER_BUTTON; then
        sleep 1s
        if grep -q $POWER_BUTTON_ACTIVE_VALUE $POWER_BUTTON; then
		    echo "Power fail detected!"
		    do_notify_and_shutdown
	    fi		
	fi
done

############################################################################
#	END OF SCRIPT
############################################################################

